@@ -200,6 +200,7 @@ int ProductFunction::calculate(MathStructure &mstruct, const MathStructure &varg
 		else mstruct.replace(vars[i], ((UnknownVariable*) vars[i])->interval());
 		vars[i]->destroy();
 	}
+	if(!check_recursive_depth(mstruct)) return 0;
 	return 1;
 
 }
@@ -745,9 +746,11 @@ bool test_functions_comparison(const MathStructure &m, const EvaluationOptions &
 	for(size_t i = 0; i < m.size(); i++) {
 		if(test_functions_comparison(m[i], eo)) return true;
 	}
-	if(m.isFunction() && m.function()->subtype() == SUBTYPE_USER_FUNCTION) {
+	if(m.isFunction() && m.function()->id() == FUNCTION_ID_REPLACE && m.size() >= 2 && (m[0].containsType(STRUCT_COMPARISON, false, true, true) > 0 || m[2].containsType(STRUCT_COMPARISON, false, true, true) > 0)) {
+		return true;
+	} else if(m.isFunction() && m.function()->subtype() == SUBTYPE_USER_FUNCTION) {
 		CALCULATOR->beginTemporaryStopMessages();
-		MathStructure mtest;
+		MathStructure mtest(m);
 		mtest.eval(eo);
 		CALCULATOR->endTemporaryStopMessages();
 		if(mtest.containsType(STRUCT_COMPARISON, false, true, true) > 0) return true;
