@@ -90,7 +90,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 			}
 			for(size_t i = 0; i < units.size(); i++) {
 				Unit *ui = units[i];
-				if(ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u  && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
+				if(ui->isActive() && ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
 					AliasUnit *aui = (AliasUnit*) ui;
 					int priority_i = aui->mixWithBase();
 					if(((priority_i > 0 && (!best_u || priority_i <= best_priority)) || (best_priority == 0 && priority_i == 0 && ((muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER && aui->expression().find_first_not_of(NUMBERS) == string::npos) || muc == MIXED_UNITS_CONVERSION_FORCE_ALL))) && (aui->mixWithBaseMinimum() <= 1 || nr.isGreaterThanOrEqualTo(aui->mixWithBaseMinimum()))) {
@@ -168,7 +168,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 				int best_priority = 0;
 				for(size_t i = 0; i < units.size(); i++) {
 					Unit *ui = units[i];
-					if(ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
+					if(ui->isActive() && ui->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) ui)->firstBaseUnit() == u && ((AliasUnit*) ui)->firstBaseExponent() == 1) {
 						AliasUnit *aui = (AliasUnit*) ui;
 						int priority_i = aui->mixWithBase();
 						if(aui->expression().find("1" DIVISION) == 0 && aui->expression().find_first_not_of(NUMBERS DIVISION, 2) == string::npos && ((priority_i > 0 && (!best_u || priority_i <= best_priority)) || (best_priority == 0 && priority_i == 0 && (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER || muc == MIXED_UNITS_CONVERSION_FORCE_ALL)))) {
@@ -699,7 +699,7 @@ bool contains_angle_ratio(const MathStructure &m) {
 	return num && den;
 }
 
-MathStructure get_units_for_parsed_expression(const MathStructure *parsed_struct, Unit *to_unit, const EvaluationOptions &eo, const MathStructure *mstruct) {
+MathStructure get_units_for_parsed_expression(const MathStructure *parsed_struct, Unit *to_unit, const EvaluationOptions &eo, const MathStructure *mstruct, string parsed_text) {
 	CompositeUnit *cu = NULL;
 	if(to_unit->subtype() == SUBTYPE_COMPOSITE_UNIT) cu = (CompositeUnit*) to_unit;
 	if(cu && cu->countUnits() == 0) return m_zero;
@@ -718,6 +718,13 @@ MathStructure get_units_for_parsed_expression(const MathStructure *parsed_struct
 		}
 		EvaluationOptions eo2 = eo;
 		if(eo.approximation == APPROXIMATION_EXACT) eo2.approximation = APPROXIMATION_TRY_EXACT;
+		if(parsed_text.length() > 2 && to_unit->baseUnit()->referenceName() == "s") {
+			size_t i = parsed_text.rfind(":");
+			if(i != string::npos && i > 0) {
+				Unit *hour = CALCULATOR->getActiveUnit(parsed_text.rfind(":", i - 1) != string::npos ? "h" : "min");
+				if(hour) return hour;
+			}
+		}
 		MathStructure munit(to_unit);
 		munit.unformat();
 		if(b_angle && (b_ratio || (!cu && to_unit->baseExponent() == 1) || (cu && cu->get(1, &exp1) && exp1 == 1))) {
@@ -732,6 +739,9 @@ MathStructure get_units_for_parsed_expression(const MathStructure *parsed_struct
 	}
 	return m_zero;
 }
+MathStructure get_units_for_parsed_expression(const MathStructure *parsed_struct, Unit *to_unit, const EvaluationOptions &eo, const MathStructure *mstruct) {
+	return get_units_for_parsed_expression(parsed_struct, to_unit, eo, mstruct, "");
+}
 
 bool contains_part_of_unit(const MathStructure &m, Unit *u) {
 	if(u->subtype() == SUBTYPE_COMPOSITE_UNIT) {
@@ -806,6 +816,49 @@ bool angle_convert(MathStructure &m, Unit *u, const EvaluationOptions &eo) {
 	return b;
 }
 
+int contains_angle_unit_cc(const MathStructure &m) {
+	if(m.isUnit() && m.unit()->baseUnit() == CALCULATOR->getRadUnit() && m.unit()->baseExponent() == 1) {
+		if(m.unit()->referenceName() == "turn") return 3;
+		if(m.unit() == CALCULATOR->getRadUnit()) return 1;
+		return 2;
+	}
+	int ret = 0;
+	for(size_t i = 0; i < m.size(); i++) {
+		if(!m.isFunction() || !m.function()->getArgumentDefinition(i + 1) || m.function()->getArgumentDefinition(i + 1)->type() != ARGUMENT_TYPE_ANGLE) {
+			int i_ret = contains_angle_unit_cc(m[i]);
+			if(i_ret > ret) ret = i_ret;
+		}
+	}
+	return ret;
+}
+int contains_non_angle_unit_cc(const MathStructure &m) {
+	if(m.isUnit()) return m.unit() != CALCULATOR->getRadUnit();
+	for(size_t i = 0; i < m.size(); i++) {
+		if(contains_non_angle_unit_cc(m[i])) return true;
+	}
+	return false;
+}
+bool remove_rad_unit_cc(MathStructure &m, const EvaluationOptions &eo) {
+	if(m.isUnit()) {
+		if(m.unit() == CALCULATOR->getRadUnit()) {
+			m.set(1, 1, 0, true);
+			return true;
+		}
+	} else {
+		bool b = false;
+		for(size_t i = 0; i < m.size(); i++) {
+			if((!m.isFunction() || !m.function()->getArgumentDefinition(i + 1) || m.function()->getArgumentDefinition(i + 1)->type() != ARGUMENT_TYPE_ANGLE) && remove_rad_unit_cc(m[i], eo)) {
+				b = true;
+			}
+		}
+		if(b) {
+			m.calculatesub(eo, eo, false);
+			return true;
+		}
+	}
+	return false;
+}
+
 MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo, bool always_convert, bool convert_to_mixed_units, bool transform_orig, MathStructure *parsed_struct) {
 	CompositeUnit *cu = NULL;
 	if(to_unit->subtype() == SUBTYPE_COMPOSITE_UNIT) cu = (CompositeUnit*) to_unit;
@@ -1207,6 +1260,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 						exp = 1;
 					}
 				}
+				if(!b_ratio && !b_angle && contains_angle_unit_cc(mstruct_new) == 3) mstruct_new.convert(getRadUnit(), false, NULL, true, eo2);
 				if(cu) {
 					MathStructure *mstruct_cu = new MathStructure(cu->generateMathStructure(false, eo.keep_prefixes));
 					Prefix *p = NULL;
@@ -1306,9 +1360,9 @@ Unit *Calculator::findMatchingUnit(const MathStructure &mstruct) {
 				}
 				for(size_t i = 0; i < units.size(); i++) {
 					Unit *u = units[i];
-					if(u->subtype() == SUBTYPE_ALIAS_UNIT && u->baseUnit() == u_base && ((AliasUnit*) u)->baseExponent() == exp) {
+					if(u->isActive() && u->subtype() == SUBTYPE_ALIAS_UNIT && u->baseUnit() == u_base && ((AliasUnit*) u)->baseExponent() == exp) {
 						return u;
-					} else if(u->subtype() == SUBTYPE_COMPOSITE_UNIT && ((CompositeUnit*) u)->countUnits() == 1 && ((CompositeUnit*) u)->get(1, &exp2)->baseUnit() == u_base && exp == ((CompositeUnit*) u)->get(1)->baseExponent(exp2)) {
+					} else if(u->isActive() && u->subtype() == SUBTYPE_COMPOSITE_UNIT && ((CompositeUnit*) u)->countUnits() == 1 && ((CompositeUnit*) u)->get(1, &exp2)->baseUnit() == u_base && exp == ((CompositeUnit*) u)->get(1)->baseExponent(exp2)) {
 						return u;
 					}
 				}
@@ -1351,10 +1405,9 @@ Unit *Calculator::findMatchingUnit(const MathStructure &mstruct) {
 				if(exp == 1) return u_base;
 				for(size_t i = 0; i < units.size(); i++) {
 					Unit *u = units[i];
-					if(u->subtype() == SUBTYPE_ALIAS_UNIT && u->baseUnit() == u_base && ((AliasUnit*) u)->baseExponent() == exp) {
+					if(u->isActive() && u->subtype() == SUBTYPE_ALIAS_UNIT && u->baseUnit() == u_base && ((AliasUnit*) u)->baseExponent() == exp) {
 						return u;
-					} else if(
-						u->subtype() == SUBTYPE_COMPOSITE_UNIT && ((CompositeUnit*) u)->countUnits() == 1 && ((CompositeUnit*) u)->get(1, &exp2)->baseUnit() == u_base && exp == ((CompositeUnit*) u)->get(1)->baseExponent(exp2)) {
+					} else if(u->isActive() && u->subtype() == SUBTYPE_COMPOSITE_UNIT && ((CompositeUnit*) u)->countUnits() == 1 && ((CompositeUnit*) u)->get(1, &exp2)->baseUnit() == u_base && exp == ((CompositeUnit*) u)->get(1)->baseExponent(exp2)) {
 						return u;
 					}
 				}
@@ -1362,7 +1415,7 @@ Unit *Calculator::findMatchingUnit(const MathStructure &mstruct) {
 			if(cu->countUnits() > 1) {
 				for(size_t i = 0; i < units.size(); i++) {
 					Unit *u = units[i];
-					if(u->subtype() == SUBTYPE_COMPOSITE_UNIT) {
+					if(u->isActive() && u->subtype() == SUBTYPE_COMPOSITE_UNIT) {
 						if(((CompositeUnit*) u)->countUnits() == cu->countUnits()) {
 							bool b = true;
 							for(size_t i2 = 1; i2 <= cu->countUnits(); i2++) {
@@ -1476,7 +1529,7 @@ Unit *Calculator::getOptimalUnit(Unit *u, bool allow_only_div, bool convert_to_l
 			Unit *best_u = NULL;
 			if(cu->countUnits() > 1) {
 				for(size_t i = 0; i < units.size(); i++) {
-					if(units[i]->subtype() == SUBTYPE_COMPOSITE_UNIT && !units[i]->isHidden() && units[i]->isSIUnit()) {
+					if(units[i]->isActive() && units[i]->subtype() == SUBTYPE_COMPOSITE_UNIT && !units[i]->isHidden() && units[i]->isSIUnit()) {
 						CompositeUnit *cu2 = (CompositeUnit*) units[i];
 						if(cu == cu2 && !cu2->isHidden()) {
 							points = max_points - 1;
@@ -1497,7 +1550,7 @@ Unit *Calculator::getOptimalUnit(Unit *u, bool allow_only_div, bool convert_to_l
 					}
 				}
 				for(size_t i = 0; i < units.size() && !best_u; i++) {
-					if(units[i]->subtype() == SUBTYPE_COMPOSITE_UNIT && !units[i]->isHidden() && units[i]->isSIUnit()) {
+					if(units[i]->isActive() && units[i]->subtype() == SUBTYPE_COMPOSITE_UNIT && !units[i]->isHidden() && units[i]->isSIUnit()) {
 						CompositeUnit *cu2 = (CompositeUnit*) units[i];
 						if(cu2->countUnits() <= cu->countUnits()) {
 							for(size_t i2 = 1; i2 <= cu2->countUnits(); i2++) {
@@ -1549,6 +1602,7 @@ Unit *Calculator::getOptimalUnit(Unit *u, bool allow_only_div, bool convert_to_l
 			AliasUnit *au;
 			for(size_t i = 0; i < units.size(); i++) {
 				u2 = units[i];
+				if(!u2->isActive()) continue;
 				if(u2->subtype() == SUBTYPE_BASE_UNIT && (points == 0 || (points == 1 && minus))) {
 					for(size_t i2 = 1; i2 <= cu->countUnits(); i2++) {
 						if(cu->get(i2, &exp)->baseUnit() == u2 && !cu->get(i2)->hasNonlinearRelationTo(u2)) {
