@@ -40,6 +40,13 @@ DataObject::DataObject(DataSet *parent_set) {
 	parent = parent_set;
 	b_uchanged = false;
 }
+DataObject::~DataObject() {
+	for(size_t i = 0; i < m_properties.size(); i++) {
+		if(m_properties[i]) {
+			m_properties[i]->unref();
+		}
+	}
+}
 
 void DataObject::eraseProperty(DataProperty *property) {
 	for(size_t i = 0; i < properties.size(); i++) {
@@ -163,6 +170,9 @@ DataProperty::DataProperty(const DataProperty &dp) {
 	m_unit = NULL;
 	set(dp);
 }
+DataProperty::~DataProperty() {
+	if(m_unit) m_unit->unref();
+}
 
 void DataProperty::set(const DataProperty &dp) {
 	stitle = dp.title(false);
@@ -376,6 +386,10 @@ DataSet::DataSet(const DataSet *o) {
 	b_loaded = false;
 	set(o);
 }
+DataSet::~DataSet() {
+	for(size_t i = 0; i < properties.size(); i++) delete properties[i];
+	for(size_t i = 0; i < objects.size(); i++) delete objects[i];
+}
 
 ExpressionItem *DataSet::copy() const {return new DataSet(this);}
 void DataSet::set(const ExpressionItem *item) {
@@ -508,10 +522,11 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 		ULONG nlang = 0;
 		DWORD n = 0;
 		if(GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &nlang, NULL, &n)) {
-			WCHAR wlocale[n];
+			WCHAR* wlocale = new WCHAR[n];
 			if(GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &nlang, wlocale, &n)) {
 				locale = utf8_encode(wlocale);
 			}
+			delete[] wlocale;
 		}
 	}
 	gsub("-", "_", locale);
